package de.mdelab.instanceGraphEditor.ui.actions;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.gef.ui.actions.ActionBarContributor;
import org.eclipse.gef.ui.actions.DeleteRetargetAction;
import org.eclipse.gef.ui.actions.RedoRetargetAction;
import org.eclipse.gef.ui.actions.UndoRetargetAction;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.actions.ActionFactory;

import de.mdelab.instanceGraphEditor.Activator;
import de.mdelab.instanceGraphEditor.ui.editor.InstanceGraphEditor;


/**
 * Contributes actions to the Editor
 * 
 * @author Phil Zoio
 */
public class InstanceGraphActionBarContributor extends ActionBarContributor
{

	FlyoutChangeLayoutAction changeLayoutAction;
	IEditorPart editor;
	
	protected void buildActions()
	{
		addRetargetAction(new UndoRetargetAction());
		addRetargetAction(new RedoRetargetAction());
		addRetargetAction(new DeleteRetargetAction());
		buildChangeLayoutAction();
		addAction(changeLayoutAction);
	}

	public void contributeToToolBar(IToolBarManager toolBarManager)
	{
		toolBarManager.add(changeLayoutAction);
	}

	private void buildChangeLayoutAction()
	{
		
		ImageDescriptor layout_img = ImageDescriptor.createFromURL(FileLocator.find(Activator.getDefault().getBundle(),new Path("icons/layout.gif"),null));
		ImageDescriptor layout_disabled_img = ImageDescriptor.createFromURL(FileLocator.find(Activator.getDefault().getBundle(),new Path("icons/layout_disabled.gif"),null));
				
		changeLayoutAction = new FlyoutChangeLayoutAction(editor);
		changeLayoutAction.setToolTipText("Automatic Layout");
		changeLayoutAction.setId("com.realpersist.gef.action.ChangeLayoutAction");
		changeLayoutAction.setImageDescriptor(layout_img);
		changeLayoutAction.setDisabledImageDescriptor(layout_disabled_img);
	}

	public void setActiveEditor(IEditorPart editor)
	{
		this.editor = editor;
		InstanceGraphEditor graphEditor = (InstanceGraphEditor) editor;
		changeLayoutAction.setActiveEditor(graphEditor);
		super.setActiveEditor(graphEditor);
		
	}

	protected void declareGlobalActionKeys()
	{
		//add support for printing
		addGlobalActionKey( ActionFactory.PRINT.getId() /*IWorkbenchActionConstants.PRINT*/);
	}


//	private static ImageDescriptor create(String iconPath, String name)
//	{
//		ImageDescriptor imgDiscr = AbstractUIPlugin.imageDescriptorFromPlugin("de.mdelab.instanceGraphEditor.ui", iconPath + name);
//		return imgDiscr;
//	}
	



}