package de.mdelab.instanceGraphEditor.ui.figures;

import org.eclipse.draw2d.ChopboxAnchor;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.LineBorder;
import org.eclipse.draw2d.ToolbarLayout;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.swt.graphics.Color;

import de.mdelab.instanceGraphEditor.ui.figures.SelfLinkAnchor.AnchorType;

public class NodeFigure extends Figure {

	private ConnectionAnchor connectionAnchor;
	
	private SelfLinkAnchor selfConnectionAnchorStart, selfConnectionAnchorEnd;
	
	public static Color tableColor = new Color(null, 255, 255, 206);

	private AttributesFigure attributesFigure = new AttributesFigure();
	
	private Label nameLabel;
	
	private Label selectedAttribute;

	public NodeFigure(Label name) {
		
		nameLabel = name;
		ToolbarLayout layout = new ToolbarLayout();
		layout.setVertical(true);
		layout.setStretchMinorAxis(true);
		setLayoutManager(layout);
		setBorder(new LineBorder(ColorConstants.black, 1));
		setBackgroundColor(tableColor);
		setForegroundColor(ColorConstants.black);
		setOpaque(true);
		name.setForegroundColor(ColorConstants.black);
		add(name);
		add(attributesFigure);
		setBounds(new Rectangle(0, 0, -1,-1));
		
	}

	public Label getNameLabel()
	{
		return nameLabel;
	}

	public AttributesFigure getAttributesFigure()
	{
		return attributesFigure;
	}

	public ConnectionAnchor getConnectionAnchor() {
		//System.out.println("figure Location when Anchor is called" + getBounds().getLocation());
		if (connectionAnchor == null) {
			connectionAnchor = new ChopboxAnchor(this);
		}
		return connectionAnchor;
	}
	
	public Label getSelectedAttribute() {
		return selectedAttribute;
	}

	public void setSelectedAttribute(Label selectedAttribute) {
		this.selectedAttribute = selectedAttribute;
	}

	
	// side either source or target. (anchor is different for source Point and target point)
	public ConnectionAnchor getSelfConnectionAnchor(AnchorType anchorType) {
		
		switch (anchorType) {
		case SOURCE_ANCHOR:
			if (selfConnectionAnchorStart == null) {
				selfConnectionAnchorStart = new SelfLinkAnchor(this, anchorType);
			}
			return selfConnectionAnchorStart;
			
		case TARGET_ANCHOR:
			if (selfConnectionAnchorEnd == null) {
				selfConnectionAnchorEnd = new SelfLinkAnchor(this, anchorType);
			}
			return selfConnectionAnchorEnd;

		default:
			if (connectionAnchor == null) {
				connectionAnchor = new ChopboxAnchor(this);
			} 
			return connectionAnchor;
		}
	}
}