package de.mdelab.sdm.interpreter.code.debug.ui.breakpoints;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.model.Breakpoint;
import org.eclipse.debug.core.model.IBreakpoint;

import de.mdelab.sdm.interpreter.code.debug.ui.SDDebugUiConstants;

public abstract class SDDebugBreakpoint extends Breakpoint {
	
	public static final String	ELEMENT_UUID		= "elementUUID";
	public static final String	ELEMENT_NAME		= "elementName";
	public static final String	ELEMENT_URI			= "elementURI";
	public static final String	EXPRESSION			= "expression";
	public static final String	EXPRESSION_LANGUAGE	= "expressionLanguage";

	private String	breakpointUUID;

	protected SDDebugBreakpoint() {
	}

	protected SDDebugBreakpoint(	final String elementUUID, 
									final String elementName,
									final String uri,
									final String expression, 
									final String expressionLanguage,
									final IResource p_resource )
	throws DebugException {
		assert elementUUID != null;

		assert expression != null && expressionLanguage != null || expression == null && expressionLanguage == null;

		breakpointUUID = null;

		final IWorkspaceRunnable runnable = new IWorkspaceRunnable() {
			
			@Override
			public void run(IProgressMonitor monitor)
			throws CoreException {
				setMarker( p_resource.createMarker( getBreakpointMarkerID() ) );//*/ResourcesPlugin.getWorkspace().getRoot().createMarker( getBreakpointMarkerID() ) );

				setAttribute( IBreakpoint.ENABLED, Boolean.TRUE );
				setAttribute(ELEMENT_UUID, elementUUID);
				setAttribute(ELEMENT_URI, uri);
				setAttribute(ELEMENT_NAME, elementName);
				setAttribute(EXPRESSION, expression);
				setAttribute(EXPRESSION_LANGUAGE, expressionLanguage);
				setAttribute(IBreakpoint.ID, getModelIdentifier() );
				setAttribute(IMarker.MESSAGE, "Element Breakpoint: " + elementUUID);
			}
		};

		run( getMarkerRule( ResourcesPlugin.getWorkspace().getRoot() ), runnable );
	}

	@Override
	public abstract String getModelIdentifier();

	protected abstract String getBreakpointMarkerID();

	public String getElementUUID()
	{
		return this.getMarker().getAttribute(ELEMENT_UUID, null);
	}

	public void setElementUUID(String elementUUID) throws CoreException
	{
		this.setAttribute(ELEMENT_UUID, elementUUID);
	}
	
	public String getElementName()
	{
		return this.getMarker().getAttribute(ELEMENT_NAME, null);
	}

	public void setElementName(String elementName) throws CoreException
	{
		this.setAttribute(ELEMENT_NAME, elementName);
	}
	
	public String getElementURI()
	{
		return this.getMarker().getAttribute(ELEMENT_URI, null);
	}

	public void setElementURI(String uri) throws CoreException
	{
		this.setAttribute(ELEMENT_URI, uri);
	}


	public String getExpressionLanguage()
	{
		return this.getMarker().getAttribute(EXPRESSION_LANGUAGE, null);
	}

	public void setExpressionLanguage(String expressionLanguage) throws CoreException
	{
		this.setAttribute(EXPRESSION_LANGUAGE, expressionLanguage);
	}

	public String getExpression()
	{
		return this.getMarker().getAttribute(EXPRESSION, null);
	}

	public void setExpression(String expression)
	throws CoreException {
		setAttribute(EXPRESSION, expression);
	}

	public String getBreakpointUUID() {
		return breakpointUUID;
	}

	public void setBreakpointUUID( final String p_breakpointUUID )
	throws CoreException {
		breakpointUUID = p_breakpointUUID;
	}
	
	public static IBreakpoint findBreakpoint( final String p_id ) {
		final IBreakpoint[] breakpoints = DebugPlugin.getDefault().getBreakpointManager().getBreakpoints( SDDebugUiConstants.DEBUG_MODEL_IDENTIFIER );
		
		for ( final IBreakpoint breakpoint : breakpoints ) {
			assert breakpoint instanceof SDDebugBreakpoint;

			if ( p_id.equals( ( (SDDebugBreakpoint) breakpoint ).getElementUUID() ) ) {
				return breakpoint;
			}
		}
		
		return null;
	}
}
