package de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.actparameter;

import org.eclipse.jface.viewers.ILabelProvider;
import org.eclipse.jface.viewers.Viewer;

import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.TableViewerComparator;

public class SDTableActivityParameterComparator extends TableViewerComparator<TableActivityParameterColumnType>  {

	/**
	 * Creates a resource sorter that will use the given sort criteria.
	 *
	 * @param criteria the sort criterion to use: one of <code>NAME</code> or 
	 *   <code>TYPE</code>
	 */
	public SDTableActivityParameterComparator( final ILabelProvider p_labelProvider ) {
		super( p_labelProvider );
		
		criteria = TableActivityParameterColumnType.NAME;
	}
	
	/* (non-Javadoc)
	 * Method declared on ViewerSorter.
	 */
	public int compare(Viewer viewer, Object o1, Object o2) {
		final SDTableActivityParameter<?, ?> param1 = (SDTableActivityParameter<?, ?>) o1;
		final SDTableActivityParameter<?, ?> param2 = (SDTableActivityParameter<?, ?>) o2;
		
		int result = 0;

		switch (criteria) {
			case NAME :
				result = compareNames( param1, param2 );
				
				break;
			
			case TYPE :
				result = compareTypes( param1, param2 );
				
				break;
			
			case INSTANCE_OBJECT :
				result = compareInstanceObjects( param1, param2 );
				
				break;
			
			default:
				result =  0;
		}
		
		if ( descending ) {
			result = - result;
		}
		
		return result;
	}

	protected int compareNames( final SDTableActivityParameter<?, ?> p_param1,
								final SDTableActivityParameter<?, ?> p_param2 ) {
		return p_param1.getName().compareTo( p_param2.getName() );
	}

	protected int compareTypes(	final SDTableActivityParameter<?, ?> p_param1,
								final SDTableActivityParameter<?, ?> p_param2 ) {
		return labelProvider.getText( p_param1.getType() ).compareTo( labelProvider.getText( p_param2.getType() ) );
	}

	protected int compareInstanceObjects(	final SDTableActivityParameter<?, ?> p_param1,
											final SDTableActivityParameter<?, ?> p_param2 ) {
		if ( p_param1.getInstanceObject() == null ) {
			return p_param2.getInstanceObject() == null ? 0 : 1;
		}

		if ( p_param2.getInstanceObject() == null ) {
			return p_param1.getInstanceObject() == null ? 0 : - 1;
		}
		
		return labelProvider.getText( p_param1.getInstanceObject() ).compareTo( labelProvider.getText( p_param2.getInstanceObject() ) );
	}
}
