package de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.instmodels;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.Table;

import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.ITableViewerListener;
import de.mdelab.sdm.interpreter.code.debug.ui.launcher.table.TextAndDialogCellEditor;

public abstract class InstanceModelEditingSupport extends EditingSupport {
	
	protected final List<ITableViewerListener<InstanceModelRow>> editorListeners;

	public InstanceModelEditingSupport( final TableViewer p_viewer ) {
	    super( p_viewer );
	    
	    editorListeners = new ArrayList<ITableViewerListener<InstanceModelRow>>();
	}

	public void addEditorListener( final ITableViewerListener<InstanceModelRow> p_listener ) {
		editorListeners.add( p_listener );
	}
	
	public boolean removeEditorListener( final ITableViewerListener<InstanceModelRow> p_listener ) {
		return editorListeners.remove( p_listener );
	}

	@Override
	protected CellEditor getCellEditor( final Object p_element ) {
//		final TextCellEditor editor = new TextCellEditor( (Table) getViewer().getControl() );
		final TextAndDialogCellEditor editor = new TextAndDialogCellEditor( (Table) getViewer().getControl() );
 
	    return editor;
	}

	@Override
	protected boolean canEdit(  final Object p_element  ) {
		return true;
	}

	@Override
	protected Object getValue(  final Object p_element  ) {
		final URI uri = getInstanceModel( (InstanceModelRow) p_element );
		
		return uri == null ? "" : uri.toString();
	}

	@Override
	protected void setValue( 	final Object p_element, 
			  					final Object p_userInputValue ) {
		final InstanceModelRow row = (InstanceModelRow) p_element;
		final URI uri;
		
		if ( p_userInputValue == null || p_userInputValue.toString().isEmpty() ) {
			uri = null;
		}
		else {
			uri = URI.createURI( p_userInputValue.toString() );
		}
		
		setInstanceModel( row, uri );
	    
	    getViewer().update( p_element, null );
	    
	    fireParameterUpdated( row );
	}
	
	protected abstract URI getInstanceModel( InstanceModelRow p_row );
	
	protected abstract void setInstanceModel( 	InstanceModelRow p_row,
												URI p_resourceUri );

	protected void fireParameterUpdated( final InstanceModelRow p_row ) {
	    for ( final ITableViewerListener<InstanceModelRow> listener : editorListeners ) {
	    	listener.rowObjectUpdated( p_row );
	    }
	}
}
